using System;
using System.Data;
using System.Windows.Forms;
using Dapfor.Net.Demo.Internal;
using Dapfor.Net.Ui;

namespace Dapfor.Net.Demo.Examples.DataBinding.MasterDetailBinding
{
    [Example("Master-detail data binding", "Data binding")]
    public partial class MasterDetailBindingForm : UserControl
    {
        public MasterDetailBindingForm()
        {
            InitializeComponent();

            //Create a demonstration table
            DataSet dataSet = CreateDataSet();


            BindingSource source1 = new BindingSource();
            BindingSource source2 = new BindingSource();

            source1.DataSource = dataSet;
            source1.DataMember = "Customers";


            source2.DataSource = source1;
            source2.DataMember = "myrelation";

            //Master table
            grid1.Headers.AutoGenerate = true;
            grid1.DataSource = source1;
            bindingNavigator1.BindingSource = source1;

            //Orders table
            grid2.Headers.AutoGenerate = true;
            grid2.DataSource = source2;
            bindingNavigator2.BindingSource = source2;

            //Initialize checkboxes
            cbCustomersAdding.Checked = grid1.DataConnection.AllowNew;
            cbOrdersAdding.Checked = grid2.DataConnection.AllowNew;
        }

        public static DataSet CreateDataSet()
        {
            // create a DataSet with one table, two columns
            DataSet dataSet = new DataSet();

            // create Customer table
            DataTable table = new DataTable("Customers");
            dataSet.Tables.Add(table);
            table.Columns.Add("customerId", typeof (int)).AutoIncrement = true;
            table.Columns.Add("name", typeof (string));
            table.PrimaryKey = new DataColumn[] {table.Columns["customerId"]};

            // create Orders table
            table = new DataTable("Orders");
            dataSet.Tables.Add(table);
            table.Columns.Add("orderId", typeof (int)).AutoIncrement = true;
            table.Columns.Add("customerId", typeof (int));
            table.Columns.Add("amount", typeof (double));
            table.PrimaryKey = new DataColumn[] {table.Columns["orderId"]};

            // create relation
            dataSet.Relations.Add("myrelation", dataSet.Tables["Customers"].Columns["customerId"], dataSet.Tables["Orders"].Columns["customerId"]);

            Random r = new Random();

            // populate the tables
            int orderId = 1;
            const int customerCount = 10;
            const int ordersPerCustomer = 5;
            for (int customerId = 1; customerId <= customerCount; customerId++)
            {
                // add customer record
                dataSet.Tables["Customers"].Rows.Add(new object[] {customerId, string.Format("customer{0}", customerId)});
            }

            for (int order = 1; order <= customerCount * ordersPerCustomer; order++)
            {
                // add order record
                dataSet.Tables["Orders"].Rows.Add(new object[] { orderId++, 1 + r.Next() % customerCount, (10 + r.Next() % 10)});
            }

            return dataSet;
        }

        private void OnGrid1FocusedRowChanged(object sender, FocusedRowEventArgs e)
        {
            //Prevent the child grid from adding while the parent table is editing
            bool hasFocus = e.NewFocusedRow != null && !e.NewFocusedRow.IsDataInserter;

            bindingNavigatorDeleteItem.Visible = e.NewFocusedRow != null && !e.NewFocusedRow.IsDataInserter;

            bindingNavigator2.Enabled = hasFocus;
            grid2.Enabled = hasFocus;
            grid2.DataConnection.AllowNew = hasFocus && cbOrdersAdding.Checked;

        }

        private void OnCustomersAddingCheckedChanged(object sender, EventArgs e)
        {
            grid1.DataConnection.AllowNew = cbCustomersAdding.Checked;
        }

        private void OnOrdersAddingCheckedChanged(object sender, EventArgs e)
        {
            grid2.DataConnection.AllowNew = cbOrdersAdding.Checked;
        }
    }
}