using System;
using System.ComponentModel;
using System.Drawing;
using System.Drawing.Design;
using System.Drawing.Drawing2D;
using System.Windows.Forms;
using System.Windows.Forms.Design;
using Dapfor.Net.Ui;

namespace Dapfor.Net.Demo.Examples.Features.ColumnFilters
{
    public class RatingFilter : UITypeEditor, IFilter
    {
        private Column column;
        private int currentRating = 1;

        #region IFilter Members

        public bool IsFiltered(Row row)
        {
            //Indicates where the row math to the column filter
            if (column != null)
            {
                Cell cell = row[column.Id];
                object value = cell != null ? cell.Value : null;
                if (value != null && value is int)
                {
                    int rating = (int) value;
                    return !(rating == currentRating);
                }
            }
            return false;
        }
        
        // Occurs when the user changes column filter.
        public event EventHandler<EventArgs> FilterUpdated { add { } remove { } }

        #endregion

        public override object EditValue(ITypeDescriptorContext context, IServiceProvider provider, object value)
        {
            //The value is a column object
            if (value is Column)
            {
                column = (Column) value;
            }

            //Get the editor service
            IWindowsFormsEditorService service = provider.GetService(typeof (IWindowsFormsEditorService)) as IWindowsFormsEditorService;
            if (service != null)
            {
                //Create a control to draw stars
                using (Control control = new Control())
                {
                    control.Width = 5 * Properties.Resources.star_grey.Width;
                    control.Height = Preferences.Grid.RowDefaultHeight;
                    control.BackColor = Color.White;

                    //Add callback to draw starts inside the control.
                    control.Paint += delegate(object sender, PaintEventArgs e)
                    {
                        //Draw a background
                        Rectangle bounds = control.Bounds;
                        using (LinearGradientBrush linearBrush = new LinearGradientBrush(bounds, Color.FromArgb(252, 205, 205), Color.FromArgb(249, 164, 164), LinearGradientMode.Vertical))
                        {
                            e.Graphics.FillRectangle(linearBrush, bounds);
                        }

                        //Draw gold and gray stars
                        bounds.Width = Properties.Resources.star_grey.Width;
                        for (int i = 1; i < 6; i++)
                        {
                            //Select image
                            Image image = i > currentRating ? Properties.Resources.star_grey : Properties.Resources.star_yellow;
                            e.Graphics.DrawImage(image, bounds);
                            bounds.X += bounds.Width;
                        }
                    };

                    //Add callback to close control when the user ends editing
                    control.Click += delegate
                    {
                        Point pt = control.PointToClient(Cursor.Position);
                        currentRating = (pt.X - control.Bounds.X) / Properties.Resources.star_grey.Width + 1;
                        service.CloseDropDown();
                    };

                    //Begin editing
                    service.DropDownControl(control);
                }
            }
            return this;
        }

        public override UITypeEditorEditStyle GetEditStyle(ITypeDescriptorContext context)
        {
            //Return the style of the editor
            return UITypeEditorEditStyle.DropDown;
        }
    }
}