using System;
using System.ComponentModel;
using System.Drawing;
using Dapfor.Net.Data;
using Dapfor.Net.Formats;

namespace Dapfor.Net.Demo.Examples.Features.CustomEditors
{
    /// <summary>
    /// Formats objects of the <see cref="Color"/> type.
    /// </summary>
    internal class CustomColorFormat : IFormat
    {
        #region IFormat Members

        public string Format(IDataField dataField)
        {
            //Get the color
            Color color = (Color) dataField.Value;
            string text = null;

            //Find corresponding known color
            foreach (KnownColor value in Enum.GetValues(typeof(KnownColor)))
            {
                Color knownColor = Color.FromKnownColor(value);

                if (knownColor.R == color.R && knownColor.G == color.G && knownColor.B == color.B && !knownColor.IsSystemColor)
                {
                    //Format the string
                    double opacity = 100 * (double)color.A/255;
                    text = string.Format("{0} [{1:0}%]", value, opacity);
                    break;
                }
            }

            //Known color is not found. Use the default formatting.
            if(string.IsNullOrEmpty(text))
            {
                text = (string)TypeDescriptor.GetConverter(typeof(Color)).ConvertTo(color, typeof(string));    
            }

            return text;
        }

        public bool CanParse(string text, IDataField dataField)
        {
            //In this example the feature is not implemented
            return false;
        }

        public void Parse(string text, IDataField dataField)
        {
        }

        #endregion
    }
}