using System;
using System.Drawing;
using System.Windows.Forms;
using System.Windows.Forms.VisualStyles;
using Dapfor.Net.Editors;
using Dapfor.Net.Ui;

namespace Dapfor.Net.Demo.Examples.Features.CustomEditors
{
    public class CustomTrackBarEditor : UITypeEditorEx
    {
        private readonly int range;

        public CustomTrackBarEditor(int range)
        {
            if (range <= 0)
            {
                throw new ArgumentException("The range must be positive", "range");
            }

            this.range = range;

        }

        public override bool GetPaintCellSupported()
        {
            //The cell should be repainted in the editor
            return true;
        }

        public override void PaintCell(PaintCellEventArgs e)
        {
            //Do basic painting without text
            e.Text = string.Empty;
            e.PaintAll();
            e.Handled = true;


            Rectangle r = e.Cell.VisibleBounds;
            r.Inflate(-4, 0);
            int value = (int) e.Cell.Value;

            Rectangle channel = new Rectangle(r.X, r.Y + r.Height/2 - 3, r.Width, 4);
            Rectangle button = new Rectangle(r.X + (value * (r.Width - 4) / range), r.Y + 3, 5, r.Height - 6);

            //Draw the track channel
            ControlPaint.DrawBorder3D(e.Graphics, channel, Border3DStyle.SunkenInner);
            
            //Draw the thumb
            if(Application.RenderWithVisualStyles)
            {
                VisualStyleRenderer renderer = new VisualStyleRenderer(VisualStyleElement.TrackBar.ThumbBottom.Hot);
                renderer.DrawBackground(e.Graphics, button);
            }
            else
            {
                ControlPaint.DrawButton(e.Graphics, button, ButtonState.Normal);
            }
        }

        public override StopEditReason EditCell(IGridEditorService service, Cell cell, StartEditReason reason)
        {
            StopEditReason stopReason = StopEditReason.Undefined;
            //Edit the cell if the user has clicked on the cell with the left button
            if (cell.Row != null && Equals(StartEditReason.LButtonClick, reason))
            {
                cell.EnsureVisible();
                //Create a real control
                using (TrackBar control = new TrackBar())
                {
                    //Initialize the control
                    control.BackColor = cell.Appearance.BackColor;
                    control.ForeColor = Color.Red;
                    control.TickStyle = TickStyle.None;
                    control.Maximum = range;
                    control.Value = (int) cell.Value;

                    //Handle the MouseUp events
                    control.MouseUp += delegate
                    {
                        //Stop edit in place
                        service.CloseCellControl(StopEditReason.Undefined);
                    };

                    //Start Edit in place. 
                    service.CellEditControl(control, cell.VirtualBounds, reason);

                    //Set a new value to Cell
                    cell.Value = control.Value;
                    stopReason = StopEditReason.UserStop;
                }
            }
            return stopReason;
        }
    }
}