using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Drawing.Design;
using Dapfor.Net.Data;
using Dapfor.Net.Formats;
using Dapfor.Net.Ui;

namespace Dapfor.Net.Demo.Examples.Features.CustomEditors
{
    /// <summary>
    /// The objects of this class will be inserted into the <see cref="Grid"/>. 
    /// </summary>
    public class DataObject
    {
        #region Fields 

        private Color color1;
        private int plusMinusValue;
        private int rating;
        private int trackValue;
        private int intValue;

        #endregion

        #region Public properties

        public DataObject(Random r)
        {
            //Populate fields with random values
            if (r != null)
            {
                rating = r.Next()%5;
                trackValue = r.Next()%11;
                plusMinusValue = r.Next()%1000;
                intValue = r.Next()%5;

                KnownColor[] kc = (KnownColor[]) Enum.GetValues(typeof (KnownColor));
                List<Color> colors = new List<Color>();
                foreach (KnownColor color in kc)
                {
                    Color c = Color.FromKnownColor(color);
                    if (!c.IsEmpty && !c.IsSystemColor)
                    {
                        colors.Add(c);
                    }
                }

                color1 = Color.FromArgb(255, colors[r.Next()%colors.Count]);
            }
        }


        [Field("rating")]
        public int Rating
        {
            get { return rating; }
            set { rating = value; }
        }

        [Field("trackbar")]
        public int TrackValue
        {
            get { return trackValue; }
            set { trackValue = value; }
        }

        [Field("plusminus")]
        [Editor(typeof (PlusMinusEditor), typeof (UITypeEditor))]
        public int PlusMibusValue
        {
            get { return plusMinusValue; }
            set { plusMinusValue = value; }
        }

        [Field("color")]
        [Format(typeof (CustomColorFormat))]
        public Color Color
        {
            get { return color1; }
            set { color1 = value; }
        }

        [Field("bullet")]
        [Editor(typeof(BulletEditor), typeof(UITypeEditor))]
        public int IntValue
        {
            get { return intValue; }
            set { intValue = value; }
        }

        #endregion
    }
}