using System;
using System.Drawing;
using System.IO;
using System.Runtime.Serialization.Formatters.Binary;
using System.Windows.Forms;
using System.Xml.Serialization;
using Dapfor.Net.Demo.Internal;
using Dapfor.Net.Ui;

namespace Dapfor.Net.Demo.Examples.Features.XmlBinarySerialization
{
    [Example("XML & binary serialization", "Features")]
    public partial class SerializationForm : UserControl
    {
        private readonly Random _r = new Random();

        public SerializationForm()
        {
            InitializeComponent();

            //Populate the grid with random data
            for (int i = 0; i < 100; ++i)
            {
                grid.Rows.Add(new object[]
                               {
                                   _r.Next(50),
                                   _r.Next(50),
                                   _r.Next(50),
                                   _r.Next(50),
                                   _r.Next(50),
                                   _r.Next(50),
                                   _r.Next(50),
                                   _r.Next(50),
                                   _r.Next(50),
                                   _r.Next(50),
                                   _r.Next(50),
                                   _r.Next(50),
                                   _r.Next(50),
                                   _r.Next(50),
                                   _r.Next(50),
                               });
            }

            RefreshColorButtons();
        }

        private void OnRowSelectorCheckedChanged(object sender, EventArgs e)
        {
            grid.RowSelector.Visible = cbRowSelector.Checked;
        }

        private void OnBtnRandomSortClick(object sender, EventArgs e)
        {
            //Random sort
            grid.Headers[0].SortedColumns.Clear();
            for (int i = _r.Next(1, 3); i >= 0; --i)
            {
                Column column = grid.Headers[0].Columns[_r.Next()%grid.Headers[0].Columns.Count];
                column.SortDirection = _r.Next()%2 == 0 ? SortDirection.Ascending : SortDirection.Descending;
            }
        }

        private void OnBtnRandomColumnsClick(object sender, EventArgs e)
        {
            //Random width and positions
            for (int i = 0; i < grid.Headers[0].Columns.Count; ++i)
            {
                Column column = grid.Headers[0].Columns[i];
                column.Width = _r.Next(70, 90);
                if (column.Visible && grid.Headers[0].VisibleColumns.Count > 0)
                {
                    column.VisibleIndex = _r.Next()%grid.Headers[0].VisibleColumns.Count;
                }
            }
        }

        private void OnBtnRandomGroupingClick(object sender, EventArgs e)
        {
            //Random grouping
            grid.Headers[0].GroupedColumns.Clear();
            for (int i = _r.Next(1, 3); i > 0; --i)
            {
                Column column = grid.Headers[0].Columns[_r.Next()%grid.Headers[0].Columns.Count];
                column.Grouped = true;
            }
        }

        private void OnBtnRandomColorsClick(object sender, EventArgs e)
        {
            //Random colors
            clrColumnPanel.Color = Color.FromArgb(_r.Next(200, 255), _r.Next(200, 255), _r.Next(200, 255));
            clrGroupPanel.Color = Color.FromArgb(_r.Next(200, 255), _r.Next(200, 255), _r.Next(200, 255));
            clrEvenRows.Color = Color.FromArgb(_r.Next(200, 255), _r.Next(200, 255), _r.Next(200, 255));
            clrOddRows.Color = Color.FromArgb(_r.Next(200, 255), _r.Next(200, 255), _r.Next(200, 255));

            OnClrColumnPanelColorChanged(this, EventArgs.Empty);
            OnClrGroupPanelColorChanged(this, EventArgs.Empty);
            OnClrEvenRowsColorChanged(this, EventArgs.Empty);
            OnClrOddRowsColorChanged(this, EventArgs.Empty);
        }

        private void OnBtnXmlSaveClick(object sender, EventArgs e)
        {
            //Save to XML file
            FileDialog fd = new SaveFileDialog();
            fd.Title = "Serialization .Net Grid to XML file";
            fd.SupportMultiDottedExtensions = false;
            fd.DefaultExt = "xml";
            fd.Filter = "XML files (*.xml)|*.xml|All files (*.*)|*.*";
            if (fd.ShowDialog() == DialogResult.OK)
            {
                using (FileStream fs = new FileStream(fd.FileName, FileMode.Create))
                {
                    XmlSerializer s = new XmlSerializer(typeof (Grid.GridSerializationState));
                    s.Serialize(fs, grid.SerializationState);
                    fs.Flush();
                }
            }
        }

        private void OnBtnXmlLoadClick(object sender, EventArgs e)
        {
            //Load from XML file
            FileDialog fd = new OpenFileDialog();
            fd.Title = "Deserialization .Net Grid from XML file";
            fd.SupportMultiDottedExtensions = false;
            fd.DefaultExt = "xml";
            fd.Filter = "XML files (*.xml)|*.xml|All files (*.*)|*.*";
            if (fd.ShowDialog() == DialogResult.OK)
            {
                using (FileStream fs = new FileStream(fd.FileName, FileMode.Open))
                {
                    XmlSerializer s = new XmlSerializer(typeof (Grid.GridSerializationState));
                    grid.SerializationState = (Grid.GridSerializationState) s.Deserialize(fs);
                }
                RefreshColorButtons();
            }
        }

        private void OnBtnBinarySaveClick(object sender, EventArgs e)
        {
            //Save to binary file
            FileDialog fd = new SaveFileDialog();
            fd.Title = "Serialization .Net Grid to binary file";
            fd.SupportMultiDottedExtensions = false;
            fd.DefaultExt = "bin";
            fd.Filter = "Binary files (*.bin)|*.bin|All files (*.*)|*.*";
            if (fd.ShowDialog() == DialogResult.OK)
            {
                using (FileStream fs = new FileStream(fd.FileName, FileMode.Create))
                {
                    BinaryFormatter formatter = new BinaryFormatter();
                    formatter.Serialize(fs, grid.SerializationState);
                    fs.Flush();
                }
            }
        }

        private void OnBtnBinaryLoadClick(object sender, EventArgs e)
        {
            //Load from bynary file
            FileDialog fd = new OpenFileDialog();
            fd.Title = "Deserialization .Net Grid from binary file";
            fd.SupportMultiDottedExtensions = false;
            fd.DefaultExt = "bin";
            fd.Filter = "Binary files (*.bin)|*.bin|All files (*.*)|*.*";
            if (fd.ShowDialog() == DialogResult.OK)
            {
                using (FileStream fs = new FileStream(fd.FileName, FileMode.Open))
                {
                    BinaryFormatter formatter = new BinaryFormatter();
                    grid.SerializationState = (Grid.GridSerializationState) formatter.Deserialize(fs);
                }
                RefreshColorButtons();
            }
        }

        private void OnClrColumnPanelColorChanged(object sender, EventArgs e)
        {
            //Column panel
            grid.Headers[0].Appearance.ColumnPanel.BackColor = PaintHelper.AlphaBlend(clrColumnPanel.Color, Color.FromArgb(60, Color.White));
            grid.Headers[0].Appearance.ColumnPanel.GradientEndBackColor = clrColumnPanel.Color;
        }

        private void OnClrGroupPanelColorChanged(object sender, EventArgs e)
        {
            //Group panel
            grid.Headers[0].Appearance.GroupPanel.BackColor = PaintHelper.AlphaBlend(clrGroupPanel.Color, Color.FromArgb(60, Color.White));
            grid.Headers[0].Appearance.GroupPanel.GradientEndBackColor = clrGroupPanel.Color;
        }

        private void OnClrEvenRowsColorChanged(object sender, EventArgs e)
        {
            //Even rows
            grid.Appearance.EvenRows.BackColor = clrEvenRows.Color;
        }

        private void OnClrOddRowsColorChanged(object sender, EventArgs e)
        {
            //Odd rows
            grid.Appearance.OddRows.BackColor = clrOddRows.Color;
        }

        private void RefreshColorButtons()
        {
            //refresh colors in buttons
            clrColumnPanel.Color = grid.Headers[0].Appearance.ColumnPanel.GradientEndBackColor;
            clrGroupPanel.Color = grid.Headers[0].Appearance.GroupPanel.GradientEndBackColor;
            clrEvenRows.Color = grid.Appearance.EvenRows.BackColor;
            clrOddRows.Color = grid.Appearance.OddRows.BackColor;
        }

        private void OnBtnKeepInMemoryClick(object sender, EventArgs e)
        {
            //Add profile to combo
            cbMemoryProfile.Items.Insert(0, new Profile(grid.SerializationState));
        }

        private void OnCbMemoryProfileSelectedIndexChanged(object sender, EventArgs e)
        {
            //Apply a profile from memory
            Profile profile = cbMemoryProfile.SelectedItem as Profile;
            if (profile != null)
            {
                grid.SerializationState = profile.State;
            }
        }

        #region Nested type: Profile

        /// <summary>
        /// Helper class to keep profile and print out a string in the combobox
        /// </summary>
        private class Profile
        {
            private readonly Grid.GridSerializationState _state;
            private readonly DateTime _timestamp;

            public Profile(Grid.GridSerializationState state)
            {
                _state = state;
                _timestamp = DateTime.Now;
            }

            public Grid.GridSerializationState State
            {
                get { return _state; }
            }

            public override string ToString()
            {
                return string.Format("Saved at {0:HH:mm:ss}", _timestamp);
            }
        }

        #endregion
    }
}