using System.Collections.Generic;
using System.Data;
using System.Drawing;
using System.Windows.Forms;
using Dapfor.Net.Editors;

namespace Dapfor.Net.Demo.Examples.GridBasics.DataSourceFiltering
{
    public class ProductEditor : DropDownEditor
    {
        private readonly List<string> _products;

        public ProductEditor(DataRowCollection products)
        {
            //Create a list of distinct products
            Dictionary<string, string> dico = new Dictionary<string, string>();
            foreach (DataRow dataRow in products)
            {
                string product = dataRow["product"] as string;
                if(!string.IsNullOrEmpty(product) && !dico.ContainsKey(product))
                {
                    dico.Add(product, product);
                }
            }

            _products = new List<string>(dico.Keys);
            _products.Sort();
        }

        public override void Populate(ListBox listBox, object value, ref ContentAlignment alignment)
        {
            //Populate the drop down list
            foreach (string product in _products)
            {
                listBox.Items.Add(product);
            }
        }

        public override object EndEdit(ListBox listBox, object value)
        {
            //Return the selected item
            if (listBox.SelectedItem != null)
            {
                return listBox.SelectedItem;
            }
            return value;
        }
    }
}