using System;
using System.Collections.Generic;
using System.Drawing;
using System.Windows.Forms;
using Dapfor.Net.Demo.Internal;
using Dapfor.Net.Ui;

namespace Dapfor.Net.Demo.Examples.GridBasics.Highlighting
{
    [Example("Highlighting", ".Net Grid basics")]
    public partial class HighlightingForm : UserControl
    {
        private readonly IList<TestClass> _dataObjects = new List<TestClass>();
        private readonly Random _random = new Random();
        private readonly Timer _timer = new Timer();

        public HighlightingForm()
        {
            InitializeComponent();

            CreateData();

            PopulateGrid();
           
            cbEnabled.Checked = grid.Highlighting.Enabled;
            cbFading.Checked = grid.Highlighting.Fading;
            udInterval.Value = grid.Highlighting.Interval.Seconds;
            udRefresh.Value = (decimal) grid.Highlighting.RefreshInterval.TotalMilliseconds;
            btnColor.Color = grid.Highlighting.Color;
            tbTransparency.Value = grid.Highlighting.Color.A/10;

            _timer.Interval = 10;
            _timer.Tick += OnTick;
            _timer.Start();
        }

        private void CreateData()
        {
            for (int i = 0; i < 100; ++i)
            {
                TestClass dataObject = new TestClass();
                dataObject.Field1 = _random.Next(50);
                dataObject.Field2 = _random.Next(50);
                dataObject.Field3 = _random.Next(50);
                dataObject.Field4 = _random.Next(50);
                dataObject.Field5 = _random.Next(50);
                dataObject.Field6 = _random.Next(50);
                dataObject.Field7 = _random.Next(50);
                dataObject.Field8 = _random.Next(50);
                dataObject.Field9 = _random.Next(50);
                dataObject.Field10 = _random.Next(50);
                _dataObjects.Add(dataObject);
            }
        }

        private void PopulateGrid()
        {
            foreach (TestClass dataObject in _dataObjects)
            {
                grid.Rows.Add(dataObject);
            }
        }

        #region Private event handlers

        private void OnTick(object sender, EventArgs e)
        {
            //dataObjects is immutable object. We can do not protect it.
            TestClass dataObject = _dataObjects[_random.Next(_dataObjects.Count)];

            //Update random data object fields
            switch (_random.Next(10))
            {
                case 0:
                    dataObject.Field1 = _random.Next(50);
                    break;

                case 1:
                    dataObject.Field2 = _random.Next(50);
                    break;

                case 2:
                    dataObject.Field3 = _random.Next(50);
                    break;

                case 3:
                    dataObject.Field4 = _random.Next(50);
                    break;

                case 4:
                    dataObject.Field5 = _random.Next(50);
                    break;

                case 5:
                    dataObject.Field6 = _random.Next(50);
                    break;

                case 6:
                    dataObject.Field7 = _random.Next(50);
                    break;

                case 7:
                    dataObject.Field8 = _random.Next(50);
                    break;

                case 8:
                    dataObject.Field9 = _random.Next(50);
                    break;

                case 9:
                    dataObject.Field10 = _random.Next(50);
                    break;
            }
        }

        private void OnCellLeave(object sender, GridCellEventArgs e)
        {
            if (cbMouseTracking.Checked)
            {
                e.Cell.Highlight(TimeSpan.FromMilliseconds(300), Color.FromArgb(200, Color.DarkRed));
            }
        }

        private void OnEnabledCheckedChanged(object sender, EventArgs e)
        {
            grid.Highlighting.Enabled = cbEnabled.Checked;
        }

        private void OnFadingCheckedChanged(object sender, EventArgs e)
        {
            grid.Highlighting.Fading = cbFading.Checked;
        }

        private void OnNumericUpDownValueChanged(object sender, EventArgs e)
        {
            grid.Highlighting.RefreshInterval = new TimeSpan(0, 0, 0, 0, (int) udRefresh.Value);
        }

        private void OnIntervalValueChanged(object sender, EventArgs e)
        {
            grid.Highlighting.Interval = new TimeSpan(0, 0, (int) udInterval.Value);
        }

        private void OnTransparencyValueChanged(object sender, EventArgs e)
        {
            int alpha = Math.Min(10*tbTransparency.Value, 255);
            Color newColor = Color.FromArgb(alpha, grid.Highlighting.Color);
            grid.Highlighting.Color = newColor;
        }

        private void OnColorChanged(object sender, EventArgs e)
        {
            Color newColor = Color.FromArgb(grid.Highlighting.Color.A, btnColor.Color);
            grid.Highlighting.Color = newColor;
        }

        private void OnGroupingChanged(object sender, GridHeaderEventArgs e)
        {
            grid.Hierarchy.ButtonBehaviour = grid.Headers[0].GroupedColumns.Count > 0
                ? ExpansionButtonBehaviour.HideWhenNoChildren
                : ExpansionButtonBehaviour.HideAlways;
        }

        #endregion


    }
}