using System;
using System.Windows.Forms;
using Dapfor.Net.Demo.Internal;
using Dapfor.Net.Ui;

namespace Dapfor.Net.Demo.Examples.GridBasics.HitTesting
{
    [Example("Hit tests", ".Net Grid basics")]
    public partial class HitTestMainForm : UserControl
    {
        public HitTestMainForm()
        {
            InitializeComponent();

            PopulateDataObjects();
        }

        private void PopulateDataObjects()
        {
            Random random = new Random();

            //Populate with some data
            for (int i = 0; i < 20; ++i)
            {
                //Populate with top-level items
                RootClass rootObject = new RootClass(i,
                                                     random.NextDouble(),
                                                     random.Next(5),
                                                     random.Next()%2 == 0 ? RootClass.EnumExample.Internal : RootClass.EnumExample.External);
                //Add to the grid
                Row rootRow = grid.Rows.Add(rootObject);
                rootRow.Expanded = true;

                //Add ten children to each root
                for (int j = 0; j < 10; ++j)
                {
                    ChildClass childObject = new ChildClass(i,
                                                            j,
                                                            DateTime.Now + new TimeSpan(1000000000000*(random.Next()%1000)),
                                                            random.Next(5),
                                                            new TimeSpan(random.Next()));
                    //Add as a child
                    rootRow.Add(childObject);
                }
            }
        }

        #region Private event handlers

        private void OnGridMouseMove(object sender, MouseEventArgs e)
        {
            //UI element type test
            tbUIElement.Text = grid.HitTest(e.Location).ToString();
            
            //Row hit test
            Row row = grid.HitTests.RowTest(e.Location);
            if (row != null)
            {
                tbRowIndex.Text = row.VisibleIndex.ToString();
                tbVisibleChildIndex.Text = row.VisibleChildIndex.ToString();
                tbDataObject.Text = row.DataObject != null ? row.DataObject.ToString() : string.Empty;
            }
            else
            {
                tbRowIndex.Text = string.Empty;
                tbVisibleChildIndex.Text = string.Empty;
                tbDataObject.Text = string.Empty;
            }

            //Column hit test
            Column column = grid.HitTests.ColumnTest(e.Location);
            if(column != null)
            {
                tbColumnId.Text = column.Id;
                tbColumnIndex.Text = column.Index.ToString();
                tbColumnVisibleIndex.Text = column.VisibleIndex.ToString();

            }
            else
            {
                tbColumnId.Text = string.Empty;
                tbColumnIndex.Text = string.Empty;
                tbColumnVisibleIndex.Text = string.Empty;
            }

            //Cell hit test
            Cell cell = grid.HitTests.CellTest(e.Location);
            if(cell != null)
            {
                tbCellValue.Text = cell.Column != null && cell.Value != null ? cell.Value.ToString() : string.Empty;
                tbCellDataObject.Text = cell.Row.DataObject != null ? cell.Row.DataObject.ToString() : string.Empty;
                tbCellColumnId.Text = cell.Column != null ? cell.Column.Id : string.Empty;
            }
            else
            {
                tbCellValue.Text = string.Empty;
                tbCellDataObject.Text = string.Empty;
                tbCellColumnId.Text = string.Empty;
            }
        }

        private void OnGridMouseLeave(object sender, EventArgs e)
        {
            tbUIElement.Text = String.Empty;
        }
        #endregion
    }
}
