using System.IO;
using Dapfor.Net.Data;

namespace Dapfor.Net.Demo.Examples.Miscellaneous.FileExplorer
{
    //Represents a directory in the file system
    internal class DirectoryObject : IExplorable
    {
        private readonly IExplorable _baseExplorable;
        private readonly bool _expandable;
        private readonly string _name;

        public DirectoryObject(string name, IExplorable baseExplorable)
        {
            _name = name;
            _baseExplorable = baseExplorable;

            try
            {
                DirectoryInfo di = new DirectoryInfo(baseExplorable.FullName + name);
                _expandable = di.GetDirectories("*", SearchOption.TopDirectoryOnly).Length > 0 || di.GetFiles("*").Length > 0;
            }
            catch
            {
                _expandable = false;
            }
        }

        [Field("Name")]
        public string DirectoryName
        {
            get { return _name; }
        }

        #region IExplorable Members

        public string FullName
        {
            get { return _baseExplorable.FullName + _name + "\\"; }
        }

        public bool Expandable
        {
            get { return _expandable; }
        }

        #endregion
    }
}