using System;
using System.ComponentModel;
using System.Drawing;
using System.Globalization;
using System.IO;
using System.Reflection;
using System.Windows.Forms;
using Dapfor.Net.Demo.Internal;

namespace Dapfor.Net.Demo
{
    internal static class Program
    {
        public static readonly string DocumentationPath = Path.Combine(Path.GetTempPath(), @"Dapfor\Grid.Demo");

        /// <summary>
        /// The main entry point for the application.
        /// </summary>
        [STAThread]
        private static void Main()
        {
            Application.EnableVisualStyles();
            Application.SetCompatibleTextRenderingDefault(false);

            //Copies documentation to the temporary folder
            EnsureDocumentation();

            try
            {
                Application.Run(new MainForm());
            }
            catch (LicenseException)
            {
            }
        }

        private static void EnsureDocumentation()
        {
            //Copies documentation from embedded resources to the temporary directory
            try
            {
                //Ensure directory
                if (!Directory.Exists(DocumentationPath))
                {
                    Directory.CreateDirectory(DocumentationPath);
                }

                //Copy images
                string basePath = typeof(Program).Namespace + ".Documentation";
                foreach (string resource in typeof(Program).Assembly.GetManifestResourceNames())
                {
                    if(resource.StartsWith(basePath, true, CultureInfo.InvariantCulture))
                    {
                        string[] s = resource.Split('.');
                        string file = s.Length > 1 ? s[s.Length - 2] + "." + s[s.Length - 1] : string.Empty;
                        string relativePath = resource.Substring(basePath.Length + 1);
                        relativePath = relativePath.Length > file.Length ? relativePath.Substring(0, relativePath.Length - file.Length - 1) : string.Empty;
                        relativePath = relativePath.Replace(".", @"\");

                        if(!string.IsNullOrEmpty(file))
                        {
                            relativePath = Path.Combine(DocumentationPath, relativePath);
                            if (!Directory.Exists(relativePath))
                            {
                                Directory.CreateDirectory(relativePath);
                            }

                            //Copy file
                            using (FileStream fs = new FileStream(Path.Combine(relativePath, file), FileMode.OpenOrCreate))
                            using (BinaryWriter bw = new BinaryWriter(fs))
                            using (Stream stream = typeof(Program).Assembly.GetManifestResourceStream(resource))
                            {
                                if (stream != null)
                                {
                                    byte[] bytes = new byte[stream.Length];
                                    stream.Read(bytes, 0, bytes.Length);
                                    bw.Write(bytes);
                                    bw.Flush();
                                }
                            }
                        }
                    }
                }
            }
            catch{}
        }
    }
}
